#include "mainwindow.h"
#include <QtGui>
#include <QtCore>
#include <string>
#include <algorithm>


void changePath(QByteArray & binary,const std::string &key,const std::string &value)
{
    QByteArray::iterator it = binary.begin();
    QByteArray::iterator end = binary.end();
    it = std::search(it,end,key.begin(),key.end());

    std::string s = key + value + '\0';
    while(it != end)
    {
        std::copy(s.begin(),s.end(),it);
        ++it;
        it =  std::search(it +1,end,key.begin(),key.end());
    }
}

void patchBinary(const std::string &newPathQt,const QString & file)
{
qDebug() << file;
    if(!QFile::exists(file) || file.contains(".back"))
    {
        return;
    }

    //sav back file
    QString backpath(file);
    backpath += ".back";

    if(!QFile::exists(backpath))
    {
        QFile::copy(file, backpath);
    }
    //load binary
    QByteArray tmp;
    {
        QFile f(backpath);
        f.open(QIODevice::ReadOnly );
        tmp = f.readAll();
    }
 //modif path
    changePath(tmp,"qt_prfxpath=", newPathQt);
    changePath(tmp,"qt_docspath=", newPathQt+"/doc");
    changePath(tmp,"qt_hdrspath=", newPathQt+"/include");
    changePath(tmp,"qt_libspath=", newPathQt+"/lib");
    changePath(tmp,"qt_binspath=", newPathQt+"/bin");
    changePath(tmp,"qt_plugpath=", newPathQt+"/plugins");
    changePath(tmp,"qt_datapath=", newPathQt);
    changePath(tmp,"qt_trnspath=", newPathQt+"/translations");
    changePath(tmp,"qt_xmplpath=", newPathQt+"/examples");
    changePath(tmp,"qt_demopath=", newPathQt+"/demos");
    /*  changePath(tmp,"qt_lcnsuser=", "qt_lcnsuser=$LICENSEE"
    changePath(tmp,"qt_lcnsprod=", "qt_lcnsprod=$LICENSE_PRODUCT"*/

    //write file patched
    {
        QFile f(file);
        f.open(QIODevice::WriteOnly );
        f.write(tmp);
    }
}



void patchText(const QString &newPathQt,const QString & file)
{
qDebug() << file;
    if(!QFile::exists(file) || file.contains(".back"))
    {
        return;
    }

    //sav back file
    QString backpath(file);
    backpath += ".back";

    if(!QFile::exists(backpath))
    {
        QFile::copy(file, backpath);
    }
    //load text file
    QStringList tmp;
    {
        QFile f(backpath);
        f.open(QIODevice::ReadOnly );
        QTextStream in(&f);
        while(!in.atEnd() )
            tmp << in.readLine();
    }
    QStringList::iterator it = tmp.begin();
    QStringList::iterator end = tmp.end();
    while(it!=end)
    {
        QStringList split = it->split("=");
        if(split.size() ==2)
        {

            if
            (
                (
                    split[0].trimmed() == "QMAKESPEC_ORIGINAL"
                    ||
                    split[0].trimmed() == "QMAKESPEC"
                )
                &&
                split[1].contains("mkspecs")
            )
            {
               *it =split[0] + "=" + newPathQt + "/mkspecs" +  split[1].split("mkspecs")[1];
            }
            if
            (

               split[0].trimmed() == "QT_BUILD_TREE"
               ||
               split[0].trimmed() == "QT_SOURCE_TREE"
            )
            {
               *it =split[0] + "=" + newPathQt;
            }
        }
        ++it;
    }


    //write file patched
    {
        QFile f(file);
        f.open(QIODevice::WriteOnly );
        QTextStream out(&f);
        out << tmp.join ("\n");
    }
}

void unpatch(const QString & file)
{

    QString backpath(file);
    backpath += ".back";
    if(!QFile::exists(backpath) || file.contains(".back"))
    {
        return;
    }
    QFile::remove (file);
    QFile::copy(backpath, file );
}



MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    QWidget * w = new QWidget;
    QGridLayout * lh = new QGridLayout(w);
    {
        QLabel * label = new QLabel("Qt Path : ");
        lh->addWidget(label,0,0);

        m_leditPath = new QLineEdit;
        m_leditPath->setReadOnly(true);
        lh->addWidget(m_leditPath,0,1,1,4);

        QToolButton * tb = new QToolButton;
        tb->setText("...");
        connect(tb,SIGNAL(clicked()),this,SLOT(getQtPath()));
        lh->addWidget(tb,0,5);

        m_actionZone    = new QWidget;
        m_actionZone->setEnabled(false);
        {
            QVBoxLayout *l  = new QVBoxLayout(m_actionZone);
            QPushButton *b  = new QPushButton("patcher");
            connect(b,SIGNAL(clicked()),this,SLOT(patchQt()));
            l->addWidget(b);

            QPushButton *b2 = new QPushButton("unpatcher");
            connect(b2,SIGNAL(clicked()),this,SLOT(unpatchQt()));
            l->addWidget(b2);

            lh->addWidget(m_actionZone,1,0,1,7);
        }
    }
    setCentralWidget(w);
    
}

MainWindow::~MainWindow()
{

}
void MainWindow::getQtPath()
{
    m_actionZone->setEnabled(false);
    m_leditPath->clear();

    static QString path;
    path = QFileDialog::getExistingDirectory(this,"Qt path",path);
    if(path.isEmpty()) 
        return;

    QFileInfo test(path +"/bin");
    QFileInfo test2(path +"/lib");
    if(!test.exists() || !test2.exists())
    {
        QMessageBox::critical (this,"repertoire incorect","le repertoire ne contient pas de version de Qt.");
        return;
    }

    if(path.size()>200)
    {
        QMessageBox::critical (this,"repertoire incorect","Le chemin du repertoire ne doit pas dpasser 200 caractres.");
        return;
        
    }
    foreach(QChar c,path)
    {
        if( c >255 || c ==' ')
        {
        QMessageBox::critical (this,"repertoire incorect","le repertoire qt doit tre exclusivement compos de caractre ASCII et sans espace.");
        return;
        }
    }
    m_actionZone->setEnabled(true);
   
    m_qtPath = path;
    m_leditPath->setText(m_qtPath);
}

void MainWindow::patchQt()
{
    QProgressDialog progress("Patche en cours...", QString(), 0, 0, this);
    progress.setWindowModality(Qt::WindowModal);
    progress.show();

    //Bin dir
    QDir binDir(m_qtPath+"/bin");
    foreach(QFileInfo f, binDir.entryInfoList(QStringList()<<"qmake"<<"qmake.exe"))
    {

        QEventLoop tmpLoop;
        QFutureWatcher<void> watcher;
        connect(&watcher,SIGNAL(finished()),&tmpLoop,SLOT(quit()));
        watcher.setFuture (QtConcurrent::run(patchBinary,m_qtPath.toStdString() , f.absoluteFilePath()));
        //patchBinary(m_qtPath.toStdString() , f.absoluteFilePath());
        tmpLoop.exec();
    }
    foreach(QFileInfo f, binDir.entryInfoList (QStringList()<<"QtCore*.dll"<< "QtCore*.so"))
    {
        QEventLoop tmpLoop;
        QFutureWatcher<void> watcher;
        connect(&watcher,SIGNAL(finished()),&tmpLoop,SLOT(quit()));
        watcher.setFuture (QtConcurrent::run(patchBinary,m_qtPath.toStdString() , f.absoluteFilePath()));
       // patchBinary(m_qtPath.toStdString() ,  f.absoluteFilePath());
        tmpLoop.exec();
    }

     //lib dir
    QDir libDir(m_qtPath+"/lib");
    foreach(QFileInfo f, libDir.entryInfoList (QStringList()<<"*QtCore*.a" <<"*QtCore*.la"<< "*QtCore*.dll"<<"*QtCore*.lib" << "*QtCore*.so"))
    {

        QEventLoop tmpLoop;
        QFutureWatcher<void> watcher;
        connect(&watcher,SIGNAL(finished()),&tmpLoop,SLOT(quit()));
        watcher.setFuture (QtConcurrent::run(patchBinary,m_qtPath.toStdString() , f.absoluteFilePath()));
       // patchBinary(m_qtPath.toStdString() ,  f.absoluteFilePath());
        tmpLoop.exec();
    }

    patchText(m_qtPath , m_qtPath + "/mkspecs/default/qmake.conf");

    patchText(m_qtPath , m_qtPath + "/.qmake.cache");


    QMessageBox::information (this,"","Application du patch termin");
}
void MainWindow::unpatchQt()
{
     QProgressDialog progress("Suppression du patche en cours...", QString(), 0, 0, this);
    progress.setWindowModality(Qt::WindowModal);
    progress.show();
    //Bin dir
    QDir binDir(m_qtPath+"/bin");
    foreach(QFileInfo f, binDir.entryInfoList(QStringList()<<"qmake"<<"qmake.exe"))
    {
        QEventLoop tmpLoop;
        QFutureWatcher<void> watcher;
        connect(&watcher,SIGNAL(finished()),&tmpLoop,SLOT(quit()));
        watcher.setFuture (QtConcurrent::run(unpatch,f.absoluteFilePath()));
       // ( f.absoluteFilePath());
        tmpLoop.exec();
        
    }
    foreach(QFileInfo f, binDir.entryInfoList (QStringList()<<"*QtCore*.dll"<< "*QtCore*.so"))
    {
       QEventLoop tmpLoop;
        QFutureWatcher<void> watcher;
        connect(&watcher,SIGNAL(finished()),&tmpLoop,SLOT(quit()));
        watcher.setFuture (QtConcurrent::run(unpatch,f.absoluteFilePath()));
       // ( f.absoluteFilePath());
        tmpLoop.exec();
    }

     //lib dir
    QDir libDir(m_qtPath+"/lib");
    foreach(QFileInfo f, libDir.entryInfoList (QStringList()<<"*QtCore*.a" <<"*QtCore*.la"<< "*QtCore*.dll"<<"*QtCore*.lib" << "*QtCore*.so"))
    {
        QEventLoop tmpLoop;
        QFutureWatcher<void> watcher;
        connect(&watcher,SIGNAL(finished()),&tmpLoop,SLOT(quit()));
        watcher.setFuture (QtConcurrent::run(unpatch,f.absoluteFilePath()));
       // ( f.absoluteFilePath());
        tmpLoop.exec();
    }

    unpatch( m_qtPath + "/mkspecs/default/qmake.conf");
    unpatch( m_qtPath + "/.qmake.cache");
    QMessageBox::information (this,"","Patch enlev");
}

